import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { Table } from 'antd';
import Search from './search';
import { calcPropertyMeta, calcGlobalMeta } from '../../utils/meta';

class TableEx extends Component {

  static initColumns(props) {
    return (props.columns || []).map((column, index) => {
      const filter = props.filters ? props.filters[index] : null;
      return {
        key: column.dataIndex,
        filtered: !!filter,
        filterDropdownVisible: false,
        filter,
      };
    });
  }
  constructor(props, context) {
    super(props, context);
    const columns = TableEx.initColumns(props);
    this.state = {
      columns,
    };
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.columns !== this.props.columns) {
      const columns = TableEx.initColumns(nextProps);
      this.setState({
        columns,
      });
    }
  }

  makeColumns() {
    if (!this.props.columns) {
      return [];
    } else {
      const { parsedMeta, renderCell } = this.props;
      return this.props.columns.map((column, index) => {
        const ret = {
          ...column,
        };
        const state = this.state.columns[index];
        if (ret.filterType) {
          const onSearch = (filter) => {
            const newColumns = [...this.state.columns];
            newColumns[index].filter = filter;
            newColumns[index].filtered = !!filter;
            newColumns[index].filterDropdownVisible = false;
            this.setState({ columns: newColumns }, () => {
              this.props.onFilter(this.state.columns.map(c => ({
                key: c.key,
                filter: c.filter,
              })));
            });
          };
          ret.filterDropdown = (
            <Search type={ret.filterType} onSearchSubmit={onSearch} filterEnums={ret.filterEnums} />
          );
          ret.filterDropdownVisible = state.filterDropdownVisible;
          ret.onFilterDropdownVisibleChange = (visible) => {
            const newColumns = [...this.state.columns];
            newColumns[index].filterDropdownVisible = visible;
            this.setState({ columns: newColumns });
          };
          ret.filterIcon = Search.getIcon(ret.filterType, state.filtered);
        }
        const orgRender = ret.render;
        ret.render = (text, record, rowIndex) => {
          if (column.dataIndex !== undefined && column.dataIndex !== null) {
            return (orgRender || renderCell)(text, record, rowIndex, calcPropertyMeta(column.dataIndex, parsedMeta, record), calcGlobalMeta(parsedMeta, record), column.dataIndex);
          } else {
            return (orgRender || renderCell)(text, record, rowIndex, null, calcGlobalMeta(parsedMeta, record), column.dataIndex);
          }
        };
        return ret;
      }, this);
    }
  }
  render() {
    return (
      <Table {...this.props} columns={this.makeColumns()} />
    );
  }
}

const funcVoid = () => {};

TableEx.propTypes = {
  onFilter: PropTypes.func,
  parsedMeta: PropTypes.shape({
    global: PropTypes.shape({
      columnData: PropTypes.object,
      rowData: PropTypes.objectOf(PropTypes.string),
    }),
    properties: PropTypes.objectOf(PropTypes.shape({
      columnData: PropTypes.object,
      rowData: PropTypes.objectOf(PropTypes.string),
    })),
  }),
  renderCell: PropTypes.func,
};
TableEx.defaultProps = {
  onFilter: funcVoid,
  // (cellData, rowData, rowIndex, meta, globalMeta, propertyName) => string|React node
  renderCell: data => `${data || ''}`,
};

export default TableEx;

